@@ -7,7 +7,7 @@ import (
 	"strings"
 	. "testing"
 
-	errors "golang.org/x/xerrors"
+	"errors"
 
 	"github.com/mediocregopher/radix/v3/resp"
 	"github.com/stretchr/testify/assert"
@@ -39,10 +39,13 @@ func TestPeekAndAssertPrefix(t *T) {
 
 		debugArgs := []interface{}{"%d) %q (got err:%s)", i, test.in, err}
 		assert.IsType(t, test.exp, err, debugArgs...)
-		if expDiscarded, ok := test.exp.(resp.ErrDiscarded); ok {
-			discarded, _ := err.(resp.ErrDiscarded)
+
+		if expDiscarded := (resp.ErrDiscarded{}); errors.As(test.exp, &expDiscarded) {
+			var discarded resp.ErrDiscarded
+			assert.True(t, errors.As(err, &discarded), debugArgs...)
 			assert.IsType(t, expDiscarded.Err, discarded.Err, debugArgs...)
 		}
+
 		if test.exp != nil {
 			assert.Equal(t, test.exp.Error(), err.Error(), debugArgs...)
 		}
@@ -103,7 +106,8 @@ func TestRESPTypes(t *T) {
 		umr := reflect.New(reflect.TypeOf(et.in).Elem())
 		um, ok := umr.Interface().(resp.Unmarshaler)
 		if !ok {
-			br.Discard(len(et.out))
+			_, err = br.Discard(len(et.out))
+			assert.NoError(t, err)
 			continue
 		}
 
@@ -131,7 +135,8 @@ func TestRESPTypes(t *T) {
 			umr := reflect.New(reflect.TypeOf(et.in).Elem())
 			um, ok := umr.Interface().(resp.Unmarshaler)
 			if !ok {
-				br.Discard(len(et.out))
+				_, err := br.Discard(len(et.out))
+				assert.NoError(t, err)
 				continue
 			}
 
@@ -149,7 +154,7 @@ func TestRESPTypes(t *T) {
 	}
 }
 
-// structs used for tests
+// structs used for tests.
 type testStructInner struct {
 	Foo int
 	bar int
@@ -459,6 +464,7 @@ func TestAnyUnmarshal(t *T) {
 			// Err
 			{in: "-ohey\r\n", out: "", shouldErr: "ohey"},
 			{in: "-ohey\r\n", out: nil, shouldErr: "ohey"},
+			{in: "-ohey\r\n", preloadEmpty: true, out: interface{}(nil), shouldErr: "ohey"},
 
 			// Int
 			{in: ":1024\r\n", out: "1024"},
@@ -616,8 +622,9 @@ func TestAnyUnmarshal(t *T) {
 
 func TestRawMessage(t *T) {
 	rmtests := []struct {
-		b     string
-		isNil bool
+		b       string
+		isNil   bool
+		isEmpty bool
 	}{
 		{b: "+\r\n"},
 		{b: "+foo\r\n"},
@@ -632,6 +639,7 @@ func TestRawMessage(t *T) {
 		{b: "$8\r\nfoo\r\nbar\r\n"},
 		{b: "*2\r\n:1\r\n:2\r\n"},
 		{b: "*-1\r\n", isNil: true},
+		{b: "*0\r\n", isEmpty: true},
 	}
 
 	// one at a time
@@ -642,6 +650,7 @@ func TestRawMessage(t *T) {
 			require.Nil(t, rm.MarshalRESP(buf))
 			assert.Equal(t, rmt.b, buf.String())
 			assert.Equal(t, rmt.isNil, rm.IsNil())
+			assert.Equal(t, rmt.isEmpty, rm.IsEmptyArray())
 		}
 		{
 			var rm RawMessage
@@ -729,3 +738,31 @@ func TestErrorAs(t *T) {
 		assert.Equal(t, *err, errDiscarded.Err)
 	}
 }
+
+func TestUnmarshalSimpleStringFieldsIntoStruct(t *T) {
+	type foo struct {
+		A int
+		B string
+	}
+
+	exp := func() foo { return foo{1, "2"} }
+
+	ins := map[string]string{
+		"simple-strings": "*4\r\n" +
+			"+A\r\n" + ":1\r\n" +
+			"+B\r\n" + "$1\r\n2\r\n",
+
+		"bulk-strings": "*4\r\n" +
+			"$1\r\nA\r\n" + ":1\r\n" +
+			"$1\r\nB\r\n" + "$1\r\n2\r\n",
+	}
+
+	for name, in := range ins {
+		t.Run(name, func(t *T) {
+			br := bufio.NewReader(bytes.NewBufferString(in))
+			var rcv foo
+			assert.NoError(t, Any{I: &rcv}.UnmarshalRESP(br))
+			assert.Equal(t, exp(), rcv)
+		})
+	}
+}
